/*
/*#
# ***************************************************************
# *                  Tabular - SIMUSOL PROYECT                  *
# *                                                             *
# *   Table extension for SIMUSOL PROYECT, for use with:        *                
# *       +  C++ Mathematical Expression Toolkit Library        *
# *          which is located in URL:                           *
# *          http://www.partow.net/programming/exprtk/index.html*
# *          used in Siru/Simusol                               *
# *       +  SCEPTRE (System for Circuit Evaluation             *
# *          and Prediction of Transient Radiation Effects)     *
# *          http://alpha.fh-friedberg.de/iem/84.html           *
# *          used in Simusol                                    *
# *   The extension was developed by                            *
# *   Diego.Saravia@ututo.org, 2016                             * 
# *   and could be used in the same terms of the toolkit        *
# *   or every free license with copyleft needed.               *
# *   http://www.simusol.org                                    *
# ***************************************************************
#
# This file is part of Tabular
#
#    Tabular is free software: you can redistribute it and/or modify
#    it under the terms of
#
#    1)the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    2) Any other free licence with copyleft recognized by the FSF
#
#    Tabular is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tabular.  If not, see <http://www.gnu.org/licenses/>.
*/

#define _GNU_SOURCE   // asprintf is not posix
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <limits.h>
#include <libgen.h>

#include "../src/simusol_table.h"

// put in a abs2rel.h
 char* abs2rel(const char *path,
	       const char *base,
	       char *result,
	       const size_t size);

 
 char* rel2abs(const char *path,
	       const char *base,
	       char *result,
	       const size_t size);

// finddir tambien en summapack, para sacar dependencias se copia (check lo mismo)
int finddir(char ** dir);


int finddir(char ** dire){
  char buf[300];
  char bufe[300];
  char *buf2;
  char *pc;
  char *buf3;
  char *pc2;
  if (readlink("/proc/self/exe", buf, sizeof(buf)) < 0){
        perror("readlink() error");
	exit;
  }
 else {
   pc=strdup(buf);
   buf2=dirname(pc);
   pc2=strdup(buf2);
   int nada=asprintf(dire,"%s/",dirname(pc2));
   //printf("readlink() returned '%s' for '%s'- %s\n", buf, "/proc/self/exe",*dire);
 }
} 

int main(void) {
    double result;   
    char *dir;
    char *gdir1 =NULL;
    char *gdir2 =NULL;
    finddir(&dir);
    int error1= asprintf(&gdir1,"%sdata/test.txt",dir);
    if (error1<0){
               printf("ERROR ASPRINTF\n");
               exit(error1);
               }

    int error2=interpolate(gdir1,"UNO","DOS",510.,2, &result);
    if (error2 <0){
               printf("ERROR INTERPOLATE test.txt\n");
	       exit(error2);
               }
    printf("---RESULTADO %d, %g\n",2,result);

     int error3= asprintf(&gdir2,"%sdata/tabla1.txt",dir);
    if (error3<0){
               printf("ERROR ASPRINTF\n");
               exit(error3);
               }
    
    int error4=interpolate(gdir2,"TIEMPO","RADIACION",25550.,2, &result);
    if (error4 <0){
               printf("ERROR INTERPOLATE tabla1.txt\n");
	       exit(error4);
               }
    printf("---RESULTADO %d, %g\n",2,result);

    printf("-----------------------------------------------------------\n\n");

    error2=interpolate(gdir1,"UNO","DOS",520.,2, &result);
    if (error2 <0){
               printf("ERROR INTERPOLATE test.txt\n");
	       exit(error2);
               }
    printf("---RESULTADO %d, %g\n",2,result);    
    printf("-----------------------------------------------------------\n\n");
    //test();
    
    error2=interpolate_file_num(gdir1,0,1,540.,2, &result);
    if (error2 <0){
               printf("ERROR INTERPOLATE test.txt\n");
	       exit(error2);
               }
    printf("---RESULTADO_FILE_NUM %d, %g\n",2,result);

    printf("-----------------------------------------------------------\n\n");


   
    error2=interpolate_num(2,0,2,530.,2, &result);
    if (error2 <0){
               printf("ERROR INTERPOLATE test.txt\n");
	       exit(error2);
               }
    printf("---RESULTADO_NUM %d, %g\n",2,result);

    printf("Examples of use of abs2rel\n");

    #define MAXPATHLEN 1000
    char resultc[MAXPATHLEN];
 
    abs2rel("/usr/src/inex", "/usr/local/lib", resultc, MAXPATHLEN);
    printf("path1 = %s\n", resultc);
    abs2rel("/usr/src", "/usr", resultc, MAXPATHLEN);
    printf("path2 = %s\n", resultc);
    abs2rel("/usr/src", "/usr/src", resultc, MAXPATHLEN);
    printf("path3 = %s\n", resultc);
    //(Result of above code)
    //path1 = ../../src
    //path2 = src
    //path3 = .
 
    rel2abs("../../src/sys", "/usr/local/lib", resultc, MAXPATHLEN);
    printf("path1 = %s\n", resultc);
    rel2abs("src/sys", "/usr", resultc, MAXPATHLEN);
    printf("path2 = %s\n", resultc);
    rel2abs(".", "/usr/src/sys", resultc, MAXPATHLEN);
    printf("path3 = %s\n", resultc);
    rel2abs(".", "/", resultc, MAXPATHLEN);
    printf("path3 = %s\n", resultc);
    //(Result of above program)
    //path1 = /usr/src/sys
    //path2 = /usr/src/sys
    //path3 = /usr/src/sys

    char *getcwd(char *buf, size_t size); 
    //EN simusol_table con malloc
 
    char cwd[1024];
    if (getcwd(cwd, sizeof(cwd)) != NULL)
       fprintf(stdout, "Current working dir: %s\n", cwd);
    else
       perror("getcwd() error");
  
    rel2abs("../data",cwd, resultc, MAXPATHLEN);
    printf("path3 = %s\n", resultc);   
    }
