/*#
# ***************************************************************
# *                  Tabular - SIMUSOL PROYECT                  *
# *                                                             *
# *   Table extension for SIMUSOL PROYECT, for use with:        *                
# *       +  C++ Mathematical Expression Toolkit Library        *
# *          which is located in URL:                           *
# *          http://www.partow.net/programming/exprtk/index.html*
# *          used in Siru/Simusol                               *
# *       +  SCEPTRE (System for Circuit Evaluation             *
# *          and Prediction of Transient Radiation Effects)     *
# *          http://alpha.fh-friedberg.de/iem/84.html           *
# *          used in Simusol                                    *
# *   The extension was developed by                            *
# *   Diego.Saravia@ututo.org, 2016                             * 
# *   and could be used in the same terms of the toolkit        *
# *   or every free license with copyleft needed.               *
# *   http://www.simusol.org                                    *
# ***************************************************************
#
# This file is part of Tabular
#
#    Tabular is free software: you can redistribute it and/or modify
#    it under the terms of
#
#    1)the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    2) Any other free licence with copyleft recognized by the FSF
#
#    Tabular is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tabular.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <stddef.h>
//#include <unistd.h>

#include <sys/types.h>
// ver ssize_t
//#if __BITS_PER_LONG != 64
//typedef int         ssize_t;
//#else
//typedef long        ssize_t;
//#endif
#include <limits.h> /* PATH_MAX */
#include <ctype.h>
#include "simusol_table.h"
#include "simusol_table_internal.h"

#define _GNU_SOURCE
//#define _POSIX_C_SOURCE 2


// put in a abs2rel.h
 char* abs2rel(const char *path,
	       const char *base,
	       char *result,
	       const size_t size);

 
 char* rel2abs(const char *path,
	       const char *base,
	       char *result,
	       const size_t size);


char *strdup(const char *s);
char *strsep (char **string_ptr, const char *delimiter);
//ssize_t
int getline(char **lineptr, size_t *n, FILE *stream);
FILE *popen(const char *command, const char *type);
int pclose(FILE *stream);
int close(FILE *fd);

// only  global data
struct dynlist tablelist;
int stinicial=1;


// this code reads a text file
// first line (with or without #) contains headers with column names (strings)
// # are comments
// other lines are numeric data organized in columns
// number of columns must follow number of headers

// este codigo lee un archivo de texto
// la primera fila tiene los encabezados y puede o no tener un # en su primer caracter
// los encabezados son strings:   falta: limitar caracteres!!!!
// en el resto del archivo # es comentario a partir de donde este en la linea
// las segundas filas tienen datos numericos dobles-float en forma de tabla, separados
// por espacios comas o tabs
// debe coincidir el numero de columnas en cada fila con el encabezado
// si hay mas cosas que numeros float en cada columna aborta
// no tiene, en principio, mas limite que la memoria de la maquina


//http://stackoverflow.com/questions/9210528/split-string-with-delimiters-in-c
//http://cpansearch.perl.org/src/CHORNY/Math-Spline-0.02/Spline.pm
//http://www1.udel.edu/CIS/105/tharvey/06F/examples/binSearch.c
//http://cpansearch.perl.org/src/JARW/Math-Derivative-0.01/Derivative.pm

////////////////////////////////////////////////////////////////////////////////////////////////

int add_memory(unsigned int *available, unsigned int occupy, void  ** addr,unsigned int size,
	       int iniavail, int two,  void  ** addr2,unsigned int size2){
  // Memory set and growth
  // Sometimes we need to mantain two arrays of the same lenght, in that case two=1
  // available is the memory already set
  // occupy is the memmory used
  if (*available==0){
              //printf("AMi M\n");
	      *available=iniavail;
	      if ((*addr =   malloc((unsigned int) *available*size)) == NULL){  
                      fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);
                      //a well-behaved fprintf should NOT malloc, so it can be used in this sort of context                       
                      exit(OOM);
                      }
              if (two){
                 if ((*addr2 =   malloc((unsigned int) *available*size2)) == NULL){
		      fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);                              
                      exit(OOM);
                      }
	         }	      
  }else if (occupy+1>*available){
              //printf("AMi R\n");
      	      *available= 2 * (occupy+1);
	      if ((*addr =  realloc(*addr, *available * size))==NULL){
                      fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);
                      exit(OOM);
	              }
              if (two){
                 if ((*addr2 =  realloc(*addr2, *available * size2))==NULL){
                      fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);
                      exit(OOM);
	              }	  
	         } 
              } 
  return 1;
}

////////////////////////////////////////////////////////////////////////////////////////////////

int  read_string_tables(const char *my_str_literal, unsigned int * numtable ) {
  char *strc = strdup(my_str_literal);  //We own str's memory now.
  char *token; 
  while ((token = strsep(&strc, ","))) {  
        create_search_table(token,  numtable);      //table name put inside table list
	                                            //tablelist grows
        }
  free(strc);
  return 0;
  
  //for windows
  //char *strsep(char **stringp, const char *delim) {
  // if (*stringp == NULL) { return NULL; }
  // char *token_start = *stringp;
  // *stringp = strpbrk(token_start, delim);
  // if (*stringp) {
  //   **stringp = '\0';
  //   (*stringp)++;
  // }
  // return token_start;
  //}
}

////////////////////////////////////////////////////////////////////////////////////////////////

int get_qty_tables( unsigned int *num_tables){
  *num_tables=tablelist.occupy;
  return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////

/*int get_view_info(unsigned int num_table, unsigned int num_view){
  return 0;
  }*/

////////////////////////////////////////////////////////////////////////////////////////////////

int get_table_info(unsigned int num_table, const char **table_name,unsigned int *tl, unsigned int *cl0){
  struct table *generictable; 
  generictable=tablelist.dpointers[num_table];
  *table_name=tablelist.names[num_table];
  //*nok=generictable->numeroslinok; //ava
  
  *tl=generictable->totallineas;
  *cl0=generictable->columnasporlinea[0];
  //printf("aaa %d %d  %s\n",*tl,*cl0,*table_name);
  //for (unsigned int n=0;n<*tl;n++){
    //printf("CL %d: %d %d\n",n,generictable->columnasporlinea[n],generictable->terminalinea[n]);
    // }
    //unsigned int numviews; 
    //unsigned int numdatos;
  
  return 0;
}
////////////////////////////////////////////////////////////////////////////////////////////////

/*int get_table_header_info(unsigned int num_table,unsigned int table_line,unsigned int *cl,unsigned int *tl ,unsigned int *tla ){

 }*/

////////////////////////////////////////////////////////////////////////////////////////////////

int get_table_line_info(unsigned int num_table,unsigned int table_line,unsigned int *cl,unsigned int *tl ,unsigned int *tla ){
  struct table *generictable; 
  generictable=tablelist.dpointers[num_table];
  *cl=generictable->columnasporlinea[table_line]; //superflueo
  *tl=generictable->terminalinea[table_line];
  *tla=generictable->terminalinea[table_line-1]; //verificar no cero, cero es encabezados
  //for (unsigned int i=*tla;i<*tl;i++){
    //printf("CL %d: %d %d\n",n,generictable->columnasporlinea[n],generictable->terminalinea[n]);
    // }				 
  return 0;				 
  }

////////////////////////////////////////////////////////////////////////////////////////////////


int get_numbers (const char   * filename,
		const char   * header_column_x,
		const char   * header_column_y,
		unsigned int * num_table,
		unsigned int * num_view
		){
  
  get_ini_tablelist();
   
  //unsigned int num_table=0; //sacar de occupy!  
  int error=create_search_table((const char *)filename,  num_table);  //ponerla comno const en todos lados   
  if (error<0){
     fprintf(stderr,"Error en create_search_table %s\n",filename);
     return error;
     }
 
  //printf("Create search table %s %d\n",filename, *num_table);
  //unsigned int num_view;
 
  error=create_search_view(*num_table,header_column_x,header_column_y,num_view);    
  if (error<0){
    //informed inside
    //fprintf(stderr,"Error en create_search_view %s\n",filename);
    return error;
    }

  //printf("numeros tabla %d vista %d \n", *num_table, *num_view);
  return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////


int get_numbers_num (const char   * filename,
		const int   nx,
		const int   ny,
		unsigned int * num_table,
		unsigned int * num_view
		){

  get_ini_tablelist();
   
  //unsigned int num_table=0; //sacar de occupy!  
  int error=create_search_table((const char *)filename,  num_table);  //ponerla comno const en todos lados   
  if (error<0){
     //fprintf(stderr,"Error en create_search_table %s\n",filename);
     return error;
     }
 
  //printf("Create search table %s %d\n",filename, *num_table);
  //unsigned int num_view;

  int rrr=create_search_view_num(*num_table, nx, ny , num_view);

  if (rrr<0){
    //informed inside
    fprintf(stderr,"Error en create_search_view %s\n",filename);
    return rrr;
    }

  //printf("numeros tabla %d vista %d \n", *num_table, *num_view);
  return 0;
}


////////////////////////////////////////////////////////////////////////////////////////////////

struct dynlist * get_ini_tablelist(void){

  if (stinicial){
     tablelist.occupy=0;
     tablelist.available=0;  
     stinicial=0;
     }
  return &tablelist;
}

////////////////////////////////////////////////////////////////////////////////////////////////


int load_table(struct table *tablex){
  //crea una tabla en una estructura tipo table, donde carga los datos del archivo
  //se la coloca como puntero en dynlist.dpointers, que es una lista de punteros a table
  
  tablex->numeroslintot=0;
  tablex->datostam=0;
  //definir todos esos numeros como unsigned
 
  tablex->numdatos=0;                                      // numero total de datos 
  //tablex->numeroslinok=1;                                  // numero de encabezados
   
  tablex->numviews=0;
  tablex->numviewsavailable=0;
  tablex->lineasavailable=0;

  //printf("FILA %s \n",tablex->file);
  // ABRIR ARCHIVO PARA GENERAR UNA TABLA 

  int cerrar=0;
  FILE *fp;
    
  // OJO NO AVISA SI HAY MUCHAS COMAS juntass O CON ESPACIOS, CON ESPACIOS ESTA BIEN REPETIRLOS CON COMAS NO    
  if (strcmp(tablex->file,"-")==0){
          fp=stdin;
	  //  printf("CASOFI STDIN\n");
  }else{
         cerrar=1;
         fp=fopen(tablex->file, "r");
	 
         if (fp==NULL) {
	      fprintf (stderr,"ERROR FO en abrir \"%s\"  at \"%s\": %d exit code %d\n",tablex->file, __FILE__, __LINE__,FO);
	      return FO;                                                // ojo no exit para que el programa controle eso.
              }
         printf("CASOFI FILE %s\n",tablex->file);
         }

    // string es la cadena donde se lee cada elemento de la tabla,
    // su tamaño posible va creciendo segun necesidad en todo el programa, nunca se achica
    // comienza con un char

    char* string;

    //printf("SIZE M: %d, R:  %d  RR: %d P: %p D: %s \n",sizem,sizer,2*sizer +1, string, string);

    unsigned int sizer = 0;  //available 
    unsigned int sizem = 2;  //occupy
  
    //int key;
    //char sup[2] = {'\0'};
    int ch = EOF;
    int ch2=EOF;
    double num=0;
    
    //char *line = NULL;
    //char *tmp = NULL;
    //size_t size = 0, index = 0;
    int lineas=0;

    tablex->znumeroslin=0;

    //int cero=0;
    int sigue=1;
    int caso=0;
    int cantchar=0;
    int finlinea=0;
    int casoant=0;
    int casoant2=0;
    while( sigue ){
      //cero=0;
      add_memory( &(sizer), sizem, (void *) &( string),sizeof(char),3,0,NULL,0);
 
      ch = getc(fp);
      ch2=ch;
      /* Check if we need to stop. */
      if (ch == EOF){
          ch = 0;
	  sigue=0;
	  caso=1;
      }else if (ch == '\n'){
          ch = 0;
	  sigue=1;
          sizem=2;
  	  caso=2;
	  
	  //	  printf("FINN\n");
	  // if (finlinea){
	  //  finlinea=0;
	  //  caso=8;            //solo con esto basta para eliminar el error de fila 0
	  //}
      }else if (ch == '\r'){ //CTRL-M
          ch = 0;
	  sigue=1;
          sizem=2;
  	  caso=8;
	  //	  printf("FINR\n ");
	  //finlinea=1;
      }else if (ch == '#' && lineas==0 && sizem==0){ //ENCABEZADO CON COMENTARIO
          ch = 0;
	  sigue=1;
          sizem=2;
  	  caso=6;
      }else if (ch == '#'){  //COMENTARIO
          ch = 0;
	  sigue=1;
          sizem=2;
  	  caso=2;
      }else if (ch == ','){
          ch = 0;
	  sigue=1;
	  sizem=2;
	  caso=3;
     }else if (ch== ' ' || ch == '\t' ){
          ch = 0;
	  sigue=1;
	  sizem=2;
	  caso=13;
	  
      } else if (ch == 0){    // cualquier cosa no letra     no da
	  sigue=1;
	  sizem=2;
	  caso=5;
      } else {
	//cero=1;
	  caso=4;
      }
      sizem++;
      
      //printf("%c",ch2);
 
      if (caso==4){   //lea un caracter
	string[sizem-3] = (char) ch;
	string[sizem-2] = '\0';
	//printf("STRING %s - %d \n",string,sizem-2);

      }else if(caso==8){ //haga como que no esta ese caracter	  //aunque baste con lo de arriba igual dejo
      }else{ //procese un dato	
	add_memory( &(tablex->datostam), tablex->numdatos, (void *) &( tablex->datos),sizeof(double),5,1,(void *)&(tablex->lineadatos),sizeof(int));

	unsigned int sl=0;
	if ((sl=strlen(string))){

	  if (lineas==0) {	      
               //int res=sscanf(string,"%s",&campo);//no hay nada que escanear
	   
	       add_memory( &(tablex->numeroslintot), tablex->znumeroslin, (void *) &( tablex->encabezados),sizeof(char*),1,0,NULL,0);
	       //(char **) tablex->encabezados; 	    
	       char *head;
	       head=strdup(string);
               //printf("ENCABEZADO: %s - %d - %d - %d - %p\n",head, tablex->znumeroslin,lineas,sl,&(tablex->encabezados[tablex->znumeroslin]));
	       tablex->encabezados[tablex->znumeroslin] = head;

               //for(int i = 0; i < tablex->znumeroslin+1; i++) {
	       //	      printf("%d: %s - %d \n",i,tablex->encabezados[i],tablex->znumeroslin);	      	 
               //      }
	    
	  }else{
  	      char u=0;
	      num=0; //poner valor nodefinido
	      int res=sscanf(string,"%lf%c",&num,&u);
	      if (res==1){ // lee ok numero
		 tablex->datos[tablex->numdatos]=num;
                 tablex->lineadatos[tablex->numdatos++]=lineas;
 	         //printf("CADENA: %s - %f - %d - %d\n",string, num,tablex->znumeroslin,lineas);
	      }else if (res==2){ //>1 es lo mismo solo lee uno mas
		 fprintf(stderr,"WARNING CADENA: %s  hay mas despues del num, comienza con: %c - %f - %d - %d\n",string,u, num,tablex->znumeroslin,lineas); //en el dato  primero un numero pero queda otraa cosa
		 tablex->datos[tablex->numdatos]=num;
                 tablex->lineadatos[tablex->numdatos++]=lineas;
	      }else{
		 fprintf(stderr,"WARNING CADENA: %s , no hay numero - %d - %d\n",string,tablex->znumeroslin,lineas);    
	         }         
	      }
	 
	}else{
	  //esto no sucede, solo lo pongo por si sucediese, se evita poniendo casoanterior
	  //printf("ERROR Longuitud CERO\n"); //pasa si hay espacios o comas continuadas  , no debiera ser para comas si para espacios
	  //es incorrecto
	  
	  /*  if (lineas==0) {
               add_memory( &(tablex->numeroslintot), tablex->znumeroslin, (void *) &( tablex->encabezados),sizeof(char*),1,0,NULL,0);
	       char *head="";	    
               //printf("ENCABEZADO: %s - %d - %d - %d - %p\n",head, tablex->znumeroslin,lineas,sl,&(tablex->encabezados[tablex->znumeroslin]));
	       tablex->encabezados[tablex->znumeroslin] = head;
	    }else{
	      tablex->datos[tablex->numdatos]=0;
              tablex->lineadatos[tablex->numdatos++]=lineas;
	      } 
	  */
	  }
	*string = '\0';
	sizem=2;  //parece demas, ya estaria antes, pero no, porque?????
	//strcpy(string,"");
      }
	
      if (caso==2){
	  if (casoant==13 || casoant==3 || (casoant==8) && (casoant2==13 || casoant2==3 )){  // para  los espacios al final de la linea
	        tablex->znumeroslin--;
	 	}
	
          tablex->totallineas=lineas+1;
	  add_memory( &(tablex->lineasavailable), tablex->totallineas, (void *) &(tablex-> columnasporlinea),
			           sizeof(unsigned int),1,1,(void *)&(tablex-> terminalinea),sizeof(unsigned int));
	  if (lineas==0){
            tablex->terminalinea[lineas]=tablex->znumeroslin;
	    tablex->columnasporlinea[lineas]=tablex->znumeroslin+1;	  //sumar una???
	  }else if (lineas==1){
	    tablex->terminalinea[lineas]=tablex->numdatos-1;
	    tablex->columnasporlinea[lineas]=tablex->numdatos; 	  //sumar una???
	  }else{
	    tablex->terminalinea[lineas]=tablex->numdatos-1;
	    tablex->columnasporlinea[lineas]=tablex->numdatos-1-tablex->terminalinea[lineas-1] ;	  //sumar una???
	    }
	  //	  printf("LLL %d %d %d  %d %d\n",lineas,tablex->znumeroslin,tablex->numdatos,tablex->terminalinea[lineas],tablex->columnasporlinea[lineas]);
  if (lineas>0){ 
          //	  printf("EEEEEEEEEEEEEEEE %d %d %d \n",lineas,tablex->znumeroslin,tablex->columnasporlinea[0]-1);
 	  if (tablex->znumeroslin != tablex->columnasporlinea[0]-1){
		    fprintf(stderr,"WARNING, linea %d con diferentes numeros %d != %d cantchar %d\n",lineas,tablex->znumeroslin,
			     tablex->columnasporlinea[0]-1,cantchar); //OK: STRING >%s< NCHAR %d, ,string,ch
        	    // aprecen muchos errores opas en file con formato DOS, ver y sacar, cuenta mal lineas
	          }else{
                    //printf ("********** linea %d terminada \n",linea);
		    }
	          }
	  
	  tablex->znumeroslin=0;
	  lineas++;	 
      }else if (caso==3){
	  tablex->znumeroslin++;
      }else if (caso==13){
	if (casoant!=13){
	  tablex->znumeroslin++;
	  }
      }   
      cantchar++;
      casoant2=casoant;
      casoant=caso;
      
    }
    if (cerrar){fclose(fp);}    
    //printf("td2 %d %d %p %p\n",tablex->numdatos,tablex->numeroslinok,&tablex->numdatos,&tablex->numeroslinok);
    return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////


int print_table(unsigned int numtable){
   //imprime tabla

   struct table *tablex=tablelist.dpointers[numtable]; 
   //printf("tds %d %d\n",tablex->numdatos,tablex->numeroslinok);
 
   printf ("  E || "); 
   for(unsigned int i = 0; i < tablex->columnasporlinea[0]; i++) {  //revisar esta mal imprime mas encabezados
     printf("%3d:  %10s | ",i,tablex->encabezados[i]);
     }
   int ld=0;
   for(unsigned int i = 0; i < tablex->numdatos; i++) {
      if (ld !=tablex->lineadatos[i]){
        printf ("\n%3d || ",tablex->lineadatos[i]);
	ld=tablex->lineadatos[i];
        }
      printf("%3d:  %10.2e | ",i,tablex->datos[i]);  //ojo esto no respeta sin limites por los enteros, arreglar
      }
      printf ("\n");
      //printf("FINNNNNNNNNN\n");

      return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////

int create_search_table(const char * token,unsigned int *nt){

        struct dynlist *xlist=&tablelist;   
        // ver que los token no esten previamente, si estan la tabla ya esta cargada.
        for (unsigned int i=0;i<xlist->occupy;i++){    
           //printf("TAB %d %d %s %s \n",i,xlist->occupy,xlist->names[i],token);
           if (strcmp(token,xlist->names[i])==0){
	     //tabla ya esta
	     *nt=i;
	     return 0;
	     }
           }	       
	////*****************************************  MAL horrible solo crea si no encuentra en todo el array
       add_memory( &(xlist->available), xlist->occupy, (void *) &(xlist->names),
		   sizeof(char *),1,1,(void *)&(xlist->dpointers),sizeof(struct table *));
       //xlist->names[xlist->occupy]= (const char *) token;       
       xlist->names[xlist->occupy]=strdup(token);
       //printf("\nNUEVA TABLA %s \n",xlist->names[xlist->occupy]);

       unsigned int tt=0;
       add_memory( &(tt), 0, (void *) &(xlist->dpointers[xlist->occupy]),sizeof(struct table),1,0,NULL,0);
       // solo se carga una tabla en cada posicion del vector no necesita realloc

       //       generictable->numdatos=0; //mal  
       struct table *generictable; 
	 
       generictable=xlist->dpointers[xlist->occupy];
       generictable->file=xlist->names[xlist->occupy];
 
       //printf("NUEVA TABLA %s - %d \n",tablelist.names[i], tablelist.dpointers[i]->numdatos);

        xlist->occupy++;
       //printf("FINNNNNNNNNN2\n");

        *nt=xlist->occupy-1;
	//no hace falta entonces nt, esta en occupy!!!!
	
        int error = load_table(generictable);	
        if (error!=0) {
	      //informed inside function 
	      //fprintf(stderr,"LT at \"%s\": %d exit code %d\n", __FILE__, __LINE__,LT);
              return LT;
	    }       
        return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
int binsearch(double data[], unsigned int start, unsigned int end, double key, int *rr){
  // busca los datos para interpolar

  //printf("BS %g %d %d %g %g\n",key,start,end,data[start],data[end]);
  // start siempre 0
  if (start > end){ //datos mal , ya se sabe que son crecientes, no hace falta comparar data[start] y data[end]        
	return -1;
        }
  
  if (key < data[start]){ //antes primero
       *rr= (int) start-1;     //MALLLLLLLLLLLLLLL
       //printf("LS %g %g\n",key, data[start]);
       return 0; //start-1;
  }else  if (key > data[end]) {//dsps ultimo
       *rr= (int) end;
       //printf("MS\n");
       return 0;//end;
       }
  unsigned int mid = (start + end)/2;   //round!!!
  //printf("BS %g  %d %g\n",key,mid,data[mid]);

  if (key > data[mid]){
     int error=binsearch(data, mid+1 , end, key,rr);
     if (error<0){
       //informed inside function
       //fprintf(stderr,"ERROR finalj ==0\n");
       }
     // printf("MORE\n");
     return error;
  }else if (key < data[mid]){
      int error=binsearch(data, start, mid-1 , key,rr);
      if (error<0){
	//informed inside
	//fprintf(stderr,"ERROR finalj ==0\n");
        }
      //printf("LESS\n");
     return error;     
  }else{ //equal to key  entre +1 -1
      *rr= (int) mid;
      //printf("MID\n");
    return 0;//mid;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////

int evaluate_lineal(unsigned int numtable, unsigned int numvista, double busca, double *result){
  
      struct table *tablex=tablelist.dpointers[numtable]; 
      struct view *vista;
      vista=tablex->pointers[numvista];
  
      double *tablacalcx=vista->cx;
      double *tablacalcy1=vista->cy1;
      double *tablacalcy2=vista->cy2;
      double *tablacalcy3=vista->cy3;

      int *tablacalcmeta=vista->meta;
      unsigned int finalj=vista->size;

      //interpola lineal
  
      int pdonde1=0;
      unsigned int donde1;
      unsigned int donde2;
     
      int error=binsearch(tablacalcx,0,finalj,busca,&pdonde1); // ojo en caso en que las llamadas sucesivas sean cercanas se puede acelerar
      if (error<0){
            //informed inside
	    //fprintf(stderr,"ERROR finalj ==0\n");
	    return -1;
            }
     //return
     
     if (pdonde1==-1){ //antes del comienzo      
       if (tablacalcmeta[0]>1){
          *result=tablacalcy1[0];
          //printf("BS2 oooo %d \n",tablacalcmeta[0]);
       }else{
          double uno0=tablacalcx[0];
	  double uno1=tablacalcx[1];
          double vuno0=tablacalcy3[0];
	  double vuno1=tablacalcy1[1];
	  *result= (busca-uno0)*(vuno1-vuno0)/(uno1-uno0) + vuno0;
          //printf("BS2 oooo %d %g %g %g %g \n",tablacalcmeta[0],uno0,uno1,vuno0,vuno1);
          }                 
       // printf("BINSEARCH AC %g entre %d - %d y vale %g \n",busca, -1,0,*result);
       return 0;
     }else{
       donde1=(unsigned int) pdonde1; 
       donde2=donde1+1;   //sacar donde2 y reemplazarlo por doinde1+1
       if(donde1==finalj){ // despues del final
         //donde2=-1;
	 if (tablacalcmeta[finalj]>1){
           *result=tablacalcy3[finalj];
           //printf("BS2 oooo %d \n",tablacalcmeta[0]);
         }else{
           double uno0=tablacalcx[finalj-1];
	   double uno1=tablacalcx[finalj];
           double vuno0=tablacalcy3[finalj-1];
	   double vuno1=tablacalcy1[finalj];
	   *result= (busca-uno0)*(vuno1-vuno0)/(uno1-uno0) + vuno0;
           //printf("BS2 oooo %d %g %g %g %g \n",tablacalcmeta[finalj],uno0,uno1,vuno0,vuno1);
	   } 	 
	 //printf("BINSEARCH %g entre %d - %d y vale %g \n",busca, donde1,-2,*result);
     }else if (busca==tablacalcx[donde1]){ //exacto
	  *result=tablacalcy2[donde1];
	  //         printf("BINSEARCH %g es %d y vale %g \n",busca, donde1,*result);
     }else{ //entre dos
           double uno0=tablacalcx[donde1];
	   double uno1=tablacalcx[donde2];
           double vuno0=tablacalcy3[donde1];
	   double vuno1=tablacalcy1[donde2];
	   *result= (busca-uno0)*(vuno1-vuno0)/(uno1-uno0) + vuno0;
	   //printf("BS2 oooo %d %d %g %g %g %g \n",tablacalcmeta[donde1],tablacalcmeta[donde2],uno0,uno1,vuno0,vuno1);       
           //printf("BINSEARCH %g entre %d - %d y vale %g \n",busca, donde1,donde2,*result);
           }   
     return 0;
     }
}

////////////////////////////////////////////////////////////////////////////////////////////////

// arreglar como devolver int error y pasar double como puntero a puntero argumento
int derivative2 (struct view *vista){ //,$yp1,$ypn,$n){
  // calcula las derivadas segundas para spline
 
  //    my $n=$#{$x};
  if (vista->hdy2!=0){  
    return  0;
    }

  double *x=vista->cx;
  double *y1=vista->cy1;
  //double *y2=vista->cy2;
  double *y3=vista->cy3;
  // double *dy2=vista->dy2;
  
  unsigned int n=vista->size;

  //no necesita realloc de entrada se carga todo
  double *u;
  unsigned int tt=0;
  add_memory( &(tt), 0, (void *) &(vista->dy2),sizeof(double),n+2,0,NULL,0);
  tt=0;
  add_memory( &(tt), 0, (void *) &(u),sizeof(double),n+2,0,NULL,0);   //ojo se cuelga si no es n+2, probar 1 que pasa
  
  vista->dy2[0]=0;
  u[0]=0;

  vista->dy2[n]=0;
  // cuando son distintos el comienzo y fin es cero asi que no hacen falta dos dy2
  for(unsigned int i=1; i<n; i++) {
          if (y1[i]!=y3[i]){
	     //comienza=1;
	     vista->dy2[i]=0;
       	     u[i]=0;
	  }else{
	    //comienza=0;	  
	    // recordar evaluar spline si coincide con el valor y2, aunque sea distinto y los anteriors y posteriorees iguales
	    if (x[i+1]==x[i-1]){
	      //no debiera pasar, pero ...
	      fprintf(stderr,"ERROR, x[i+1]==x[i-1], x iguales en lineas sucesivas\n");
              return -1;
	      }
            if (x[i+1]==x[i]){
	      //no debiera pasar, pero ...
	      fprintf(stderr,"WARNING, x iguales en lineas sucesivas\n");
              //return -1;
              }
	    if (x[i-1]==x[i]){
	      //no debiera pasar, pero ...
	      fprintf(stderr,"WARNING x iguales en lineas sucesivas\n");
              //return -1;
	    }
	      
            double sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
	    double p=sig*vista->dy2[i-1]+2.0; 	
	    if (p==0){
  	       fprintf(stderr,"ERROR p cero\n");
               return -1;
	       }
	    vista->dy2[i]=(sig-1.0)/p;

	    // no se usa y2 sino y1=y3  para evitar problemas
	    //u[i]=(6.0*( (y1[i+1]-y3[i])/(x[i+1]-x[i])-
	    //     (y3[i]-y3[i-1])/(x[i]-x[i-1]))/
	    //	(x[i+1]-x[i-1])-sig*u[i-1])/p;
            // algo huele muy mal en dinamarca, con la expresion anterior daba error, partido no

	    u[i]=(6.0*( (y1[i+1]-y3[i])/(x[i+1]-x[i])-
		      (y3[i]-y3[i-1])/(x[i]-x[i-1]))/
	              (x[i+1]-x[i-1])-sig*u[i-1])/p;               
            }	
         }

  unsigned int iii=n-1;
  while( iii>0){  
        //      printf("BB %g %g\n",u[i],dy2[i]);
	vista->dy2[iii]=vista->dy2[iii]*vista->dy2[iii+1]+u[iii];
	--iii;
        }
  //return @y2;

  free(u);
  vista->hdy2=1;
  
  /*for(unsigned int i = 0; i < vista->size+1; i++) {
       printf("DD %d  %g %g \n",i,x[i],vista->dy2[i]);      
       }*/
    
  //return dy2;
  return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////


//int klo es la posicion en x donde empieza el intervalo de interpolacion, usar binsearch
// v es donde se busca
int evaluate_spline(unsigned int numtable, unsigned int numvista, double busca, double *result){
  
  struct table *tablex=tablelist.dpointers[numtable]; 
  struct view *vista;
  vista=tablex->pointers[numvista];
 
  derivative2 (vista);  // meter esto en spline
  
  double *x=vista->cx;
  double *y1=vista->cy1;
  double *y2=vista->cy2;
  double *y3=vista->cy3;
  double *dy2=vista->dy2;
  int *meta=vista->meta;
  unsigned int finalj=vista->size;

  int klop=0; 
  int error= binsearch(x,0,finalj,busca,&klop);  //revisar si es
  if (error<0){
    // print inside 
    //fprintf(stderr,"ERROR finalj ==0\n");
    exit(error);
    }
  
  //printf("KLO %d %d %g\n",klo,finalj,busca);
  
  if (klop==-1){ //antes del comienzo
    // quizas se pueda hacer que la pendiente en 0 del tramo 0-1 coincida con la extrapolacion.
     if (meta[0]>1){
           *result= y1[0];
	   // printf("BS2 ACU oooo %d \n",meta[0]);
	   return 0;
     }else{
          double uno0=x[0];
	  double uno1=x[1];
          double vuno0=y3[0];
	  double vuno1=y1[1];
	  if    (uno1==uno0){
	      //no debiera pasar, pero ...
	      fprintf(stderr,"ERROR x iguales en lineas sucesivas\n");
              return -1;
	      }	   
	  *result= (busca-uno0)*(vuno1-vuno0)/(uno1-uno0) + vuno0;
	  //         printf("BS2 AC oooo %d %g %g %g %g \n",meta[0],uno0,uno1,vuno0,vuno1);
	  return 0;
     }       

  
  } else{
    unsigned int klo=(unsigned int) klop;
    if (klo==finalj){  //despues del final
       if (meta[0]>1){
           *result=y3[0];
	   //          printf("BS2 DFU oooo %d \n",meta[0]);
	   return 0;
       }else{
          double uno0=x[finalj-1];
	  double uno1=x[finalj];
          double vuno0=y3[finalj-1];
	  double vuno1=y1[finalj];
	  if    (uno1==uno0){
	      //no debiera pasar, pero ...
	      fprintf(stderr,"ERROR x iguales en lineas sucesivas\n");
	      return -1;
	      }
	  *result=(busca-uno0)*(vuno1-vuno0)/(uno1-uno0) + vuno0;

	  //         printf("BS2 DFE oooo %d %g %g %g %g %g %g\n",meta[0],uno0,uno1,vuno0,vuno1,busca,*result);
	  return 0;
       }       

  } else if (busca==x[klo]){  //exacto
    *result = y2[klo];
    //   printf("BS2 DFEXA oooo %d %g %g \n", klo, x[klo],y2[klo]);
    return 0;
  } else{
  
  double h=x[klo+1]-x[klo];
  if (h==0) {
    fprintf(stderr,"ERROR: Zero interval in spline data.\n");
    //se supone que construccion de x impide esto
    return -1;
    }
  
  double a=(x[klo+1]-busca)/h;
  double b=(busca-x[klo])/h;
 

  *result=a*y3[klo] + b*y1[klo+1]
        +((a*a*a-a)*dy2[klo]
	+(b*b*b-b)*dy2[klo+1])*(h*h)/6.0;
   
  //printf("BS2 INTER %d %g %g\n",klo,busca,*result);
  return 0;
}

  }
    
}

////////////////////////////////////////////////////////////////////////////////////////////////

int gnuplot_view(unsigned int numtable, unsigned int numvista){
		   //int meta[], double x[],double *y1,double *y2,double *y3, int N){     

   struct table *tablex=tablelist.dpointers[numtable]; 
   struct view *vista;
   
   vista=tablex->pointers[numvista];
  
   double *x=vista->cx;
   double *y1=vista->cy1;
   double *y2=vista->cy2;
   double *y3=vista->cy3;  
   unsigned int N=vista->size+1;

   //    printf("NNNNNNNNNNNNN3 %d\n",N);
   //exit(0);
  
   FILE * fp;
   if (!(fp= fopen ("gnudata.txt", "w"))){
     printf("ERROR fopen\n");
     exit;
     }

    FILE * gp;
   if (!(gp= popen ("gnuplot -persistent", "w"))){
     printf("ERROR popen\n");
     exit;
     }

  double minx=x[0];
  double maxx=x[N-1];
  double largo=maxx-minx;
  
  fprintf(gp, " plot '-' with lines, '-' with points,'-' with points lc rgb 'red' ,'-' with lines lc rgb 'black' \n");
  fprintf(fp, " plot '-' with lines, '-' with points,'-' with points lc rgb 'red' ,'-' with lines lc rgb 'black' \n");
  //fprintf(gp, "plot '-' with lines\n");
  int px=1;
  for (unsigned int i=0; i<N; i++){
       if (px==1){
          fprintf(gp, "%g %g\n", x[i],y1[i]);
	  fprintf(fp, "%g %g\n", x[i],y1[i]);
	  //printf( "GNU %g %g\n", x[i],y1[i]);
       }else{
          fprintf(gp, "%g %g\n", x[i],y3[i]);
	  fprintf(fp, "%g %g\n", x[i],y3[i]);
  	  //printf( "GNU %g %g\n", x[i],y3[i]);
          }
       if(y1[i]!=y3[i]){
	  fprintf(gp, "%g %g\n", x[i],y3[i]);
	  fprintf(fp, "%g %g\n", x[i],y3[i]);
  	  //printf( "GNU %g %g\n", x[i],y3[i]);
	  px=3;
       }else{
 	  px=1;
          }
       }
    fflush(gp);
    fflush(fp);
    fprintf(gp, "e\n");
    fprintf(fp, "e\n");
    for (unsigned int iiii=0; iiii<N; iiii++){
       fprintf(gp, "%g %g\n", x[iiii],y2[iiii]);
       fprintf(fp, "%g %g\n", x[iiii],y2[iiii]);
       }
    fflush(gp);
    fflush(fp);
    fprintf(gp, "e\n");
    fprintf(fp, "e\n");
    N=N-1;
    double result;

    double duno=minx-largo/5;
    evaluate_lineal(numtable,numvista,duno,&result);
    fprintf(gp, "%g %g\n", duno,result);
    fprintf(fp, "%g %g\n", duno,result);
    //printf( "R %g %g\n", duno,result);

    duno=(maxx-minx)/2;
    evaluate_lineal(numtable,numvista,duno,&result);
    fprintf(gp, "%g %g\n", duno,result);
    fprintf(fp, "%g %g\n", duno,result);
    //printf( "R %g %g\n", duno,result);

    duno=maxx;
    evaluate_lineal(numtable,numvista,duno,&result);
    fprintf(gp, "%g %g\n", duno,result);
    fprintf(fp, "%g %g\n", duno,result);
    //printf( "R %g %g\n", duno,result);

    duno=maxx+largo/5;
    evaluate_lineal(numtable,numvista,duno,&result);
    fprintf(gp, "%g %g\n", duno,result);
    fprintf(fp, "%g %g\n", duno,result);
    //printf( "R %g %g\n", duno,result);

    /*
    evaluate_lineal(numtable,numvista,16000, &result);
    fprintf(gp, "%g %g\n", 16000.,result);
    printf( "R %g %g\n", 10.,result);
    */

    //poner alguno con salto en test
    
    fflush(gp);
    fprintf(gp, "e\n");
    fflush(fp);
    fprintf(fp, "e\n");

    //exit(0);	   
    //double * dy2=NULL;
   
    //derivative2 (vista);

    /*for(int ii = 0; ii < N+1; ii++) {
      printf("DD %d  %g %g \n",ii,x[ii],dy2[ii]);      
      }
      */
 
    for(double dd = minx-largo/10 ; dd < maxx+largo/10; dd+=largo/1000) { 
       evaluate_spline(numtable,numvista,dd,&result);
       fprintf(gp, "%g %g\n", dd,result);
       fprintf(fp, "%g %g\n", dd,result);
       }
   
    evaluate_lineal(numtable,numvista,minx+largo/10.,&result);
     //printf(" LINEAL %g %g\n",550., result);

    evaluate_spline(numtable,numvista,minx+largo/10,&result);
     //printf(" SPLINE %g %g\n", 550.,result);

    fflush(gp);
    fprintf(gp, "e\n");
    fflush(fp);
    fprintf(fp, "e\n");
    
    // for (i=0; i<N; i++){fprintf(gp, "%g %g\n", x[i],y[i+1]);}
    //fflush(gp);
    //fprintf(gp, "e\n");  

    pclose(gp);
    close(fp);

    return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////


//tablelist pasar o no shadow?
int create_search_view(unsigned int numtable, const  char *sx, const char *sy , unsigned int *vistan){

  unsigned int nx=0;
  unsigned int ny=1;
  struct table *tablex=tablelist.dpointers[numtable];
  //struct view *v;
  //ingresar directamente *tablex a la funcion
  if (strcmp(sx,sy)==0){
          fprintf(stderr,"ERROR x e y son la misma columna= %s, error: %d \n",sx,MC); // usar formato estandar
	  return MC;
          }
  
  int cambio=0; 
  for(unsigned int i = 0; i <tablex->columnasporlinea[0]; i++) {
    printf("%3d: sx: >%s< sy: >%s< enc: >%s< \n",i,sx,sy,tablex->encabezados[i]);
    if (strcmp(sx,tablex->encabezados[i])==0){
      nx=i;
      cambio++;
    }else if (strcmp(sy,tablex->encabezados[i])==0){
      ny=i;
      cambio++;
      }
   if (cambio>1){
      //printf("salio\n");
      break;
      }
   }
   if (cambio!=2){
	  fprintf(stderr,"%s at \"%s\": %d exit code %d\n",SSC, __FILE__, __LINE__,SC);  // que SSC imprima una string con el error
	  return SC;
          }
   printf("COLUMNAS %3d:  %10s ; %3d:  %10s \n",nx,tablex->encabezados[nx],ny ,tablex->encabezados[ny]);
   int rrr=create_search_view_num(numtable, nx, ny , vistan);
   return rrr;
}

////////////////////////////////////////////////////////////////////////////////////////////////


//tablelist pasar o no shadow?
int create_search_view_num(unsigned int numtable, int nx, int ny , unsigned int *vistan){
   struct view *v;
   struct table *tablex=tablelist.dpointers[numtable];

  if (nx == ny){
          fprintf(stderr,"ERROR x e y son la misma columna= %d, error: %d \n",nx,MC); // usar formato estandar
	  return MC;
          }

  //printf ("NUMVIEWS %d\n",tablex->numviews);
  for (unsigned int i=0; i<tablex->numviews;i++){ // ya esta calculada la vista?
     v=tablex->pointers[i];                                  // 0  1   1     2
     //printf("busq vista i %d, nx %d, ny %d, anx %d,any %d\n",i,nx,ny,v->nx,v->ny);
     if (nx==v->nx && ny==v->ny){       
       *vistan=i;
       return 0;
       }
     }
  //struct view dvista;
  struct view *vista;

  unsigned int tt=0;
  add_memory( &(tt), 0, (void *) &(vista),sizeof(struct view),1,0,NULL,0);
  vista->hdy2=0;
  
  add_memory( &(tablex->numviewsavailable), tablex->numviews, (void *) &( tablex->pointers),sizeof(struct view *),1,0,NULL,0);  
  tablex->pointers[tablex->numviews]=vista;

  int ld=0;
  unsigned int col=0;
  double *tablacalcx;
  double *tablacalcy1;
  double *tablacalcy2;
  double *tablacalcy3;
  int *tablacalcmeta;
  tt=0;
  add_memory( &(tt), 0, (void *) &(tablacalcx)   ,sizeof(double),tablex->numdatos +1,0,NULL,0);  //verel +1
  tt=0;
  add_memory( &(tt), 0, (void *) &(tablacalcy1)  ,sizeof(double),tablex->numdatos +1,0,NULL,0);
  tt=0;
  add_memory( &(tt), 0, (void *) &(tablacalcy2)  ,sizeof(double),tablex->numdatos +1,0,NULL,0);
  tt=0;
  add_memory( &(tt), 0, (void *) &(tablacalcy3)  ,sizeof(double),tablex->numdatos +1,0,NULL,0);
  tt=0;
  add_memory( &(tt), 0, (void *) &(tablacalcmeta),sizeof(int)   ,tablex->numdatos +1,0,NULL,0);
  tt=0;
   
  unsigned int j=0;
  //double xant;
  vista->size=0;
  //printf("INTERPOLA %d\n",tablex->numdatos);
  unsigned int sdx=0;
  unsigned int sdy=0;

  double dx=0;
  double dy=0;
  int dz=0;
  for(unsigned int i = 0; i < tablex->numdatos; i++) {
    if (ld !=tablex->lineadatos[i]){        
       ld=tablex->lineadatos[i];
       col=0;
       sdx=0;
       sdy=0;
    }else{
       col++;
       }

    // printf("INTERPOLA %d - %d \n",i,col);
    if (col==nx){
	dx=tablex->datos[i];
	sdx=1;
    }else if (col==ny){
	dy=tablex->datos[i];
	sdy=1;
        }
    //printf("INTERPOLA %d - %d - %g -%d %d \n",i,col,tablex->datos[i],sdx,sdy);

    if (sdx && sdy){            
	sdx=0;
        sdy=0;	
        if (j==0){
	    //printf("PRIMERA %g\n",dx);
            tablacalcx[j]=dx;	
	    tablacalcy1[j]=dy;
	    tablacalcy2[j]=dy;
            tablacalcy3[j]=dy;
	    dz=1;
	    tablacalcmeta[j]=dz;
	    j++;
	}else{  	
	    if (dx<tablacalcx[j-1]){
	      fprintf(stderr,"ERROR, orden de columna x incorrecto, %d - %g - %d - %g\n", nx,dx,j,tablacalcx[j-1]);
	       return(-5);           	    
	    }else  if(  dx!=tablacalcx[j-1]){ // el punto es diferente al anterior, se ponen 3 dy  iguales
	       //printf("AVANZA %g\n",dx);
	       tablacalcx[j]=dx;	
	       tablacalcy1[j]=dy;
	       tablacalcy2[j]=dy;
               tablacalcy3[j]=dy;
	       dz=1;
	       tablacalcmeta[j]=dz;
	       j++;
	       //todo lo que sigue son para puntos distintos con dx iguales //podria usar dz no?
	    }else if (tablacalcmeta[j-1]==2){  //ya hay dos puntos con dx igual, va el tercero
	                                                // se pone el dy2 diferente
	                                                //se supone que j>1 porque son diferentes, sino como
	        //printf("TERCERO = %g\n",dx);
      	        tablacalcy2[j-1]=dy;
	        dz=3;
	        tablacalcmeta[j-1]=dz;
		
	    }else  if (tablacalcmeta[j-1]==3){ //ya hay tres con dx igual, mal  
	           fprintf(stderr,"WARNING, mas de tres puntos distintos con x igual no cambio nada, %d - %g - %d %g %g %g %g\n",
		   nx,dx,j, tablacalcy1[j-1], tablacalcy2[j-1],tablacalcy3[j-1], dy);	  
	        //return(-7);
  	    }else{                                         // el segundo va como igual al primero
	                                               // se pone el dy3 como diferente
	         //printf("SEGUNDO %g\n",dx);
	         //tablacalcy2[j-1]=dy;
                 tablacalcy3[j-1]=dy;
	         dz=2;
	         tablacalcmeta[j-1]=dz;

		 
	         }
	    }

	   if (tablacalcmeta[j-1]==2){
	      tablacalcy2[j-1]=(tablacalcy1[j-1]+tablacalcy3[j-1])/2;
   	      }
	   //printf("OOO %d %d %g : %g %g %g %d\n",j-1,ld,dx,tablacalcy1[j-1],tablacalcy2[j-1],tablacalcy3[j-1],tablacalcmeta[j-1]);     
	  
	              	       
           }
   }
   vista->size =j-1;
   /*for(int i = 0; i < finalj+1; i++) {
     printf("columns  %d  %g %g %g %g %d\n",i,tablacalcx[i],tablacalcy1[i],tablacalcy2[i],tablacalcy3[i],tablacalcmeta[i]);      
     }*/
   
   // no se agrega nada pero si hay datos extrapolados en esos casos se sigue igual 
    vista->nx=nx;
    vista->ny=ny; 
    vista->cx=tablacalcx;
    vista->cy1=tablacalcy1;
    vista->cy2=tablacalcy2;
    vista->cy3=tablacalcy3;
    vista->meta=tablacalcmeta;
    unsigned int vistann=tablex->numviews;
    tablex->numviews++;
    *vistan=vistann;
    return 0;
}     

////////////////////////////////////////////////////////////////////////////////////////////////


int interpolate(const char * file, const char* scx, const char *scy, double v, int method, double *presult){
  // file is the name of the file with the table data.
  // scx and scy are the column names in the headers, for x and y
  // v is the x value for interpolate, 
  // method: 1 is lineal interpolation, 2 is spline
  
  // v es el dato, file el archivo o tabla, x  e y las columnas, method
  // 1 es interpolacion lineal, 2 es spline
  
  unsigned int num_table=0; //sacar de occupy!
 
  unsigned int num_view;
  
  int error3=get_numbers(file,scx,scy,&num_table,&num_view);
  if (error3<0){
       // print inside
       //fprintf(stderr,"ERROR getnumbers en inter %s\n",file);
       return error3;
       }
  //printf("TABU %s \n",tablelist.names[0]);
  //printf("Getnumbers en inter %s\n",file);
  //printf("Funcion c test.\n  file: %s  x: %s y: %s v: %g m %d occup %d num_table %d vista %d\n",file,scx,scy,v,method,tablelist.occupy,num_table,num_view);
  
  if (method ==1 ){
     evaluate_lineal(num_table,num_view,v,presult);
     //printf( "e li x %g y %g\n", 10.,*presult);
   }else{
     /* for(int i = 0; i < N+1; i++) {
       printf("DD %d  %g %g \n",i,x[i],dy2[i]);      
       }*/
     evaluate_spline(num_table,num_view,v,presult);   
     }
  //printf("e %s x %g y %g\n",ss, v,*presult);

  return 1;
}

////////////////////////////////////////////////////////////////////////////////////////////////

void trim(char * s) {
    char * p = s;
    int l = strlen(p);
    while(isspace(p[l - 1])) p[--l] = 0;
    while(* p && isspace(* p)) ++p, --l;
    memmove(s, p, l + 1);
}   

////////////////////////////////////////////////////////////////////////////////////////////////

int  readfileuno(char ** fn, int si){
    // read a file etable.txt
    // with a list of lines, each line with a number and a file name 
    // donde hay una lista de numeros con su archivo respectivo,
    // linea por linea, separados el numero y el nombre del archivo por blanco

    unsigned int i=0;
    FILE *myfile;
    char * line = NULL;
    size_t len = 0;
    size_t read;
    long number=0;
    char *filename=NULL;
    unsigned num=0;
    unsigned int maximun=0;
        
    //maximun=available;
    // ver como pasar con global del sceptre el nombre del archivo en proceso a esta funcion
    myfile = fopen("etable.txt","r");
    if (myfile== NULL){
      return -1;
    }

    //error MALLOC CALLOC FOPEN!
              
    while ((read = getline(&line, &len, myfile)) != -1) {
        //printf("Retrieved line of length %zu :\n", read);
        number = strtol(line, &line, 10);
	//	number = strtok(line, " ");
	num= (unsigned int)number;

	//nu[i]=number;
	if (si){
	 filename = strdup(line); //strtok(NULL," ");
         strtok(filename, "\n");
	 trim(filename);
	 //printf("TABLE FILE >%s<\n",filename);
	    
         // requiere the real existance of file, without exits!!!!
	 //#define PATH_MAX 100000	 
         //char buf[PATH_MAX + 1]; /* not sure about the "+ 1" */ 
         //char *res = realpath("etable.txt", buf);
         //if (res) {
	 //printf("This source is at %s.\n", buf);
	 //}else{
	 //perror("realpath");
	 //   exit(EXIT_FAILURE);
	 //   }

	 #include <errno.h>
         char *getcwd(char *buf, size_t size);
         size_t size = 500;
         char *buf;
         if ((buf = (char *) malloc(size)) == NULL){ 
	       fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);
               exit(OOM);
               } 	
         char *dir = getcwd(buf, size);
         while (dir == NULL && errno == ERANGE){
           size *= 2;           
	   if ((buf = (char *) realloc(buf,size)) == NULL){
	     	  fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);
	       // pedantic if fails with 500, computer will not run
               exit(OOM);
               } 
	   dir = getcwd(buf, size);
           }       
         //printf("dat dir is: %s - %d\n", dir,(int) size);
         //free(buf);
	
         //#define MAXPATHLEN 10
	 //char resultc[MAXPATHLEN];
	 /*        char cwd[1024];
         char resultc[MAXPATHLEN];
         if (getcwd(cwd, sizeof(cwd)) != NULL){
	  //fprintf(stdout, "Current working dir: %s\n", cwd);
	  }else{
           perror("getcwd() error");
	   exit;
	   }
	 */
         size=500;
         char *buf2;
         if ((buf2 = (char *) malloc(size)) == NULL){ 
	  fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);
	       // pedantic if fails with 500, computer will not run
               exit(OOM);
               } 
	  char *RRR=rel2abs(filename,dir, buf2, size);
	  while (RRR == NULL && errno == ERANGE){
           size *= 2;
	   if ((buf2 = (char *) realloc(buf2,size)) == NULL){
	     	  fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);
	       // pedantic if fails with 500, computer will not run
               exit(OOM);
               } 
           RRR=rel2abs(filename,dir, buf2, size);
           }       
         //printf("dat dir is: %s - %d\n",buf2,(int) size);
         //free(buf2);
	 //printf("path3 = >%s< >%s< >%s< %d %d %d\n", buf2,filename,dir,errno,ERANGE,EINVAL);      	 
         fn[num]=buf2;
	
	}else{
	 if (num>maximun) {
	    //printf("MMM %d %d\n",maximun,num);
	    maximun=num;
	    //fn=calloc(maximun+100,sui);
	    };
	}	  
	//printf(" %s, %zu %d \n",fn[num],read, num);
	i++;
        }

   
    fclose(myfile);

    return maximun+1;
    }

////////////////////////////////////////////////////////////////////////////////////////////////


unsigned int readfile(char *** fn, int max){
  //horrible leo dos veces, usar realloc es mas complejo
  //porque hay que hacer anular la parte nueva y da mas trabajo pienso que leer el archivo dos veces, nose
  unsigned int sui=sizeof(char *); 
  if (max==0){
      max=readfileuno(*fn,0);
      //printf("MAX %d\n",max);
      if (max<0){
	// etablas.txt not found
        printf("ERROR: Can not open file etable.txt\n");
        printf("   Depends on the existance of file etable.txt in current dir\n");
        printf("   There is one in tabular_project_src/examples\n");	
	exit (-98); // poner código, es error de usuario no de programa o maquina
        }
      if (( *fn=calloc(max,sui)) == NULL){  
                      fprintf(stderr,"OOM at \"%s\": %d exit code %d\n", __FILE__, __LINE__,OOM);
		      //a well-behaved fprintf should NOT malloc, so it can be used in this sort of context
                      exit(OOM);
                      }      
      readfileuno(*fn,1);
      }
   return max;
   }


////////////////////////////////////////////////////////////////////////////////////////////////


int interpolate_file_num(const char * file, int nx, int ny, double v, int method, double *presult){
  // file is the name of the file with the table data.
  // x and y are the column order, for x and y
  // v is the x value for interpolate, 
  // method: 1 is lineal interpolation, 2 is spline
  
  unsigned int num_table=0; //sacar de occupy! 
  unsigned int num_view;
  
  int error3=get_numbers_num(file,nx,ny,&num_table,&num_view);
  if (error3<0){
       // print inside
       fprintf(stderr,"ERROR getnumbers en inter %s\n",file);
       return error3;
       }
  //printf("TABU %s \n",tablelist.names[0]);
  //printf("Getnumbers en inter %s\n",file);
  //printf("Funcion c test.\n  file: %s  x: %d y: %d v: %g m %d occup %d num_table %d vista %d\n",file,nx,ny,v,method,tablelist.occupy,num_table,num_view);
  
  if (method ==1 ){
     evaluate_lineal(num_table,num_view,v,presult);
     //printf( "e li x %g y %g\n", 10.,*presult);
   }else{
     /* for(int i = 0; i < N+1; i++) {
       printf("DD %d  %g %g \n",i,x[i],dy2[i]);      
       }*/    
     evaluate_spline(num_table,num_view,v,presult);   
     }
  //printf("e %s x %g y %g\n",ss, v,*presult);

  return 1;
}

////////////////////////////////////////////////////////////////////////////////////////////////


int interpolate_num(int nf, int nx, int ny, double v, int method, double *presult){    
    char ** fn;      
    unsigned int max=0;
    max=readfile(&fn,max);
    char *file = fn[nf];
    if (file){
             //printf ("Numero %d es archivo en tablas.txt: %s\n",nf,file);
             int err=interpolate_file_num(file,nx,ny, v, method,presult);	     
	     if (err <0){
	       printf("ERROR INTERPOLATE_FILE_NUM %d, RE %g\n",err,*presult);	       
	       return -1;
               }
	  }else{
             printf ("Number %d do not have a file name in etablas.txt: %s\n",nf,file);
	     return -1;
             }
    }
